@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 #include <mongoc/mongoc-util-private.h>
 #if !defined(_WIN32)
 #include <sys/types.h>
@@ -42,18 +43,6 @@
 #include "test-libmongoc.h"
 #include "TestSuite.h"
 
-/*
- * Prevent failing on pedantic GCC/clang warning: "ISO C forbids conversion of
- * function pointer to object pointer type."
- */
-#ifdef __clang__
-#pragma clang diagnostic warning "-Wpedantic"
-#elif __GNUC__ > 6
-#pragma GCC diagnostic warning "-Wpedantic"
-#elif __GNUC__ <= 6
-#pragma GCC diagnostic warning "-pedantic"
-#endif
-
 static bson_once_t once = BSON_ONCE_INIT;
 static bson_mutex_t gTestMutex;
 static TestSuite *gTestSuite;
@@ -124,6 +113,45 @@ static BSON_ONCE_FUN (_test_suite_ensure_mutex_once)
 }
 
 
+static void
+_handle_signal (int signum)
+{
+   const char *s = "\nProcess was interrupted by the delivery of a signal.\n";
+   const char *sigstr;
+   size_t n;
+   switch (signum) {
+   case SIGABRT:
+      sigstr = "SIGABRT - Abnormal termination";
+      break;
+   case SIGINT:
+      sigstr = "SIGINT - Interrupted";
+      break;
+   case SIGTERM:
+      sigstr = "SIGTERM - Termination requested";
+      break;
+   case SIGSEGV:
+      sigstr = "SIGSEGV - Access violation";
+      break;
+   default:
+      sigstr = "(Unknown signal delivered)";
+   }
+#ifdef BSON_OS_UNIX
+   /* On POSIX these APIs are signal-safe */
+   n = write (STDERR_FILENO, s, strlen (s));
+   n = write (STDERR_FILENO, "  ", 2);
+   n = write (STDERR_FILENO, sigstr, strlen (sigstr));
+   n = write (STDERR_FILENO, "\n", 1);
+   fsync (STDERR_FILENO);
+#else
+   /* On Windows these APIs are signal-safe */
+   fprintf (stderr, "\n%s\n  %s\n", s, sigstr);
+   fflush (stderr);
+#endif
+   _Exit (signum);
+   (void) n;
+}
+
+
 void
 TestSuite_Init (TestSuite *suite, const char *name, int argc, char **argv)
 {
@@ -137,8 +165,14 @@ TestSuite_Init (TestSuite *suite, const char *name, int argc, char **argv)
    suite->flags = 0;
    suite->prgname = bson_strdup (argv[0]);
    suite->silent = false;
+   suite->ctest_run = NULL;
    _mongoc_array_init (&suite->match_patterns, sizeof (char *));
 
+   suite->prev_sigabrt = signal (SIGABRT, _handle_signal);
+   suite->prev_sigint = signal (SIGINT, _handle_signal);
+   suite->prev_sigterm = signal (SIGTERM, _handle_signal);
+   suite->prev_sigsegv = signal (SIGSEGV, _handle_signal);
+
    for (i = 1; i < argc; i++) {
       if (0 == strcmp ("-d", argv[i])) {
          suite->flags |= TEST_DEBUGOUTPUT;
@@ -177,6 +211,16 @@ TestSuite_Init (TestSuite *suite, const char *name, int argc, char **argv)
       } else if ((0 == strcmp ("-s", argv[i])) ||
                  (0 == strcmp ("--silent", argv[i]))) {
          suite->silent = true;
+      } else if ((0 == strcmp ("--ctest-run", argv[i]))) {
+         if (suite->ctest_run) {
+            test_error ("'--ctest-run' can only be specified once");
+         }
+         if (argc - 1 == i) {
+            test_error ("'--ctest-run' requires an argument");
+         }
+         suite->flags |= TEST_NOFORK;
+         suite->silent = true;
+         suite->ctest_run = bson_strdup (argv[++i]);
       } else if ((0 == strcmp ("-l", argv[i])) ||
                  (0 == strcmp ("--match", argv[i]))) {
          char *val;
@@ -192,6 +236,10 @@ TestSuite_Init (TestSuite *suite, const char *name, int argc, char **argv)
       }
    }
 
+   if (suite->match_patterns.len != 0 && suite->ctest_run != NULL) {
+      test_error ("'--ctest-run' cannot be specified with '-l' or '--match'");
+   }
+
    if (test_framework_getenv_bool ("MONGOC_TEST_VALGRIND")) {
       suite->flags |= TEST_VALGRIND;
    }
@@ -251,9 +299,9 @@ TestSuite_CheckMockServerAllowed (void)
 }
 
 static void
-TestSuite_AddHelper (void *cb_)
+TestSuite_AddHelper (void *ctx)
 {
-   TestFunc cb = (TestFunc) cb_;
+   TestFunc cb = (TestFunc) ((TestFnCtx *) ctx)->test_fn;
 
    cb ();
 }
@@ -263,12 +311,8 @@ TestSuite_Add (TestSuite *suite, /* IN */
                const char *name, /* IN */
                TestFunc func)    /* IN */
 {
-   TestSuite_AddFull (suite,
-                      name,
-                      TestSuite_AddHelper,
-                      NULL,
-                      (void *) func,
-                      TestSuite_CheckDummy);
+   TestSuite_AddFullWithTestFn (
+      suite, name, TestSuite_AddHelper, NULL, func, TestSuite_CheckDummy);
 }
 
 
@@ -277,12 +321,8 @@ TestSuite_AddLive (TestSuite *suite, /* IN */
                    const char *name, /* IN */
                    TestFunc func)    /* IN */
 {
-   TestSuite_AddFull (suite,
-                      name,
-                      TestSuite_AddHelper,
-                      NULL,
-                      (void *) func,
-                      TestSuite_CheckLive);
+   TestSuite_AddFullWithTestFn (
+      suite, name, TestSuite_AddHelper, NULL, func, TestSuite_CheckLive);
 }
 
 
@@ -384,6 +424,17 @@ _TestSuite_AddFull (TestSuite *suite,  /* IN */
 }
 
 
+void
+_TestSuite_TestFnCtxDtor (void *ctx)
+{
+   TestFuncDtor dtor = ((TestFnCtx *) ctx)->dtor;
+   if (dtor) {
+      dtor (ctx);
+   }
+   free (ctx);
+}
+
+
 #if defined(_WIN32)
 static void
 _print_getlasterror_win (const char *msg)
@@ -568,6 +619,10 @@ TestSuite_RunTest (TestSuite *suite, /* IN */
 
    for (i = 0; i < test->num_checks; i++) {
       if (!test->checks[i]()) {
+         if (suite->ctest_run) {
+            /* Write a marker that tells CTest that we are skipping this test */
+            test_msg ("@@ctest-skipped@@");
+         }
          if (!suite->silent) {
             bson_string_append_printf (
                buf,
@@ -676,6 +731,8 @@ TestSuite_PrintHelp (TestSuite *suite) /* IN */
       "first error).\n"
       "    -l, --match PATTERN   Run test by name, e.g. \"/Client/command\" or "
       "\"/Client/*\". May be repeated.\n"
+      "    --ctest-run TEST      Run only the named TEST for CTest\n"
+      "                          integration.\n"
       "    -s, --silent          Suppress all output.\n"
       "    -F FILENAME           Write test results (JSON) to FILENAME.\n"
       "    -d                    Print debug output (useful if a test hangs).\n"
@@ -771,11 +828,19 @@ TestSuite_PrintJsonSystemHeader (FILE *stream)
 #endif
 }
 
-char *
+static const char *
+getenv_or (const char *name, const char *dflt)
+{
+   const char *s = getenv (name);
+   return s ? s : dflt;
+}
+
+static const char *
 egetenv (const char *name)
 {
-   return getenv (name) ? getenv (name) : "";
+   return getenv_or (name, "");
 }
+
 static void
 TestSuite_PrintJsonHeader (TestSuite *suite, /* IN */
                            FILE *stream)     /* IN */
@@ -847,7 +912,7 @@ TestSuite_PrintJsonHeader (TestSuite *suite, /* IN */
             test_framework_getenv_bool ("MONGOC_CHECK_IPV6") ? "true" : "false",
             (suite->flags & TEST_NOFORK) ? "false" : "true",
             (suite->flags & TEST_TRACE) ? "true" : "false",
-            egetenv ("MONGODB_API_VERSION"));
+            getenv_or ("MONGODB_API_VERSION", "null"));
 
    bson_free (hostname);
    bson_free (udspath);
@@ -887,14 +952,18 @@ test_matches (TestSuite *suite, Test *test)
 {
    int i;
 
+   if (suite->ctest_run) {
+      /* We only want exactly the named test */
+      return strcmp (test->name, suite->ctest_run) == 0;
+   }
+
    /* If no match patterns were provided, then assume all match. */
    if (suite->match_patterns.len == 0) {
       return true;
    }
 
    for (i = 0; i < suite->match_patterns.len; i++) {
-      char *pattern =
-         _mongoc_array_index (&suite->match_patterns, char *, i);
+      char *pattern = _mongoc_array_index (&suite->match_patterns, char *, i);
       if (TestSuite_TestMatchesName (suite, test, pattern)) {
          return true;
       }
@@ -919,6 +988,14 @@ TestSuite_RunAll (TestSuite *suite /* IN */)
       }
    }
 
+   if (suite->ctest_run) {
+      /* We should have matched *at most* one test */
+      ASSERT (count <= 1);
+      if (count == 0) {
+         test_error ("No such test '%s'", suite->ctest_run);
+      }
+   }
+
    for (test = suite->tests; test; test = test->next) {
       if (test_matches (suite, test)) {
          status += TestSuite_RunTest (suite, test, &count);
@@ -1011,12 +1088,18 @@ TestSuite_Destroy (TestSuite *suite)
 
    free (suite->name);
    free (suite->prgname);
+   free (suite->ctest_run);
    for (i = 0; i < suite->match_patterns.len; i++) {
       char *val = _mongoc_array_index (&suite->match_patterns, char *, i);
       bson_free (val);
    }
 
    _mongoc_array_destroy (&suite->match_patterns);
+
+   signal (SIGABRT, suite->prev_sigabrt);
+   signal (SIGINT, suite->prev_sigint);
+   signal (SIGTERM, suite->prev_sigterm);
+   signal (SIGSEGV, suite->prev_sigsegv);
 }
 
 
