@@ -39,24 +39,24 @@ get_node (mongoc_topology_t *topology, const char *host_and_port)
 
 
 static bool
-has_server_description (mongoc_topology_t *topology, const char *host_and_port)
+has_server_description (const mongoc_topology_t *topology,
+                        const char *host_and_port)
 {
-   mongoc_set_t *servers = topology->description.servers;
+   mc_shared_tpld td = mc_tpld_take_ref (topology);
+   const mongoc_set_t *servers = mc_tpld_servers_const (td.ptr);
    bool found = false;
    int i;
-   mongoc_server_description_t *sd;
+   const mongoc_server_description_t *sd;
 
-   bson_mutex_lock (&topology->mutex);
-   for (i = 0; i < (int) topology->description.servers->items_len; i++) {
-      sd = (mongoc_server_description_t *) mongoc_set_get_item (servers, i);
+   for (i = 0; i < (int) servers->items_len; i++) {
+      sd = mongoc_set_get_item_const (servers, i);
       if (!strcmp (sd->host.host_and_port, host_and_port)) {
          found = true;
          break;
       }
    }
 
-   bson_mutex_unlock (&topology->mutex);
-
+   mc_tpld_drop_ref (&td);
    return found;
 }
 
@@ -150,13 +150,10 @@ _test_topology_reconcile_rs (bool pooled)
    /*
     * remove server1 from set. server0 is the primary, with tags.
     */
-   RS_RESPONSE_TO_HELLO (
-      server0, 6, true, true, server0); /* server1 absent */
+   RS_RESPONSE_TO_HELLO (server0, 6, true, true, server0); /* server1 absent */
 
    BSON_ASSERT (selects_server (client, tag_read_prefs, server0));
-   bson_mutex_lock (&client->topology->mutex);
    BSON_ASSERT (!client->topology->stale);
-   bson_mutex_unlock (&client->topology->mutex);
 
    if (!pooled) {
       ASSERT_CMPINT (1, ==, debug_stream_stats.n_failed);
@@ -354,6 +351,7 @@ test_topology_reconcile_from_handshake (void *ctx)
    bson_error_t error;
    int count;
    mongoc_topology_scanner_node_t *node;
+   mc_shared_tpld td = MC_SHARED_TPLD_NULL;
    mongoc_async_cmd_t *cmd;
 
    bson_mutex_init (&data.mutex);
@@ -394,7 +392,9 @@ test_topology_reconcile_from_handshake (void *ctx)
    ASSERT_OR_PRINT (r, error);
 
    /* added server descriptions */
-   ASSERT_CMPSIZE_T (topology->description.servers->items_len, >, (size_t) 1);
+   mc_tpld_renew_ref (&td, topology);
+   ASSERT_CMPSIZE_T (mc_tpld_servers_const (td.ptr)->items_len, >, (size_t) 1);
+   mc_tpld_drop_ref (&td);
 
    /* didn't add nodes yet, since we're not in the scanner loop */
    DL_COUNT (topology->scanner->nodes, node, count);
@@ -407,7 +407,9 @@ test_topology_reconcile_from_handshake (void *ctx)
    ASSERT_CMPINT (count, ==, 0);
 
    /* allow pool to start scanner thread */
-   topology->scanner_state = MONGOC_TOPOLOGY_SCANNER_OFF;
+   bson_atomic_int_exchange (&topology->scanner_state,
+                             MONGOC_TOPOLOGY_SCANNER_OFF,
+                             bson_memory_order_seq_cst);
    mongoc_client_pool_push (pool, client);
    client = mongoc_client_pool_pop (pool);
 
@@ -500,7 +502,6 @@ test_topology_reconcile_retire_single (void)
    BSON_ASSERT (selects_server (client, secondary_read_prefs, secondary));
 
    /* remove secondary from primary's config */
-   bson_mutex_lock (&topology->mutex);
    RS_RESPONSE_TO_HELLO (primary, 6, true, false, primary);
 
    /* step 2: cluster opens new stream to primary - force new stream in single
@@ -512,7 +513,6 @@ test_topology_reconcile_retire_single (void)
       mongoc_stream_destroy (node->stream);
       node->stream = NULL;
    }
-   bson_mutex_unlock (&topology->mutex);
 
    /* step 3: run "ping" on primary, triggering a connection and handshake, thus
     * step 4 & 5: the primary tells the scanner to retire the secondary node */
